@@ -1,5 +1,5 @@
 /*  Lunzip - Decompressor for lzip files
-    Copyright (C) 2010, 2011 Antonio Diaz Diaz.
+    Copyright (C) 2010, 2011, 2012, 2013 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 /*
-    Return values: 0 for a normal exit, 1 for environmental problems
+    Exit status: 0 for a normal exit, 1 for environmental problems
     (file not found, invalid flags, I/O errors, etc), 2 to indicate a
     corrupt or invalid input file, 3 for an internal consistency error
     (eg, bug) which caused lunzip to panic.
@@ -24,14 +24,14 @@
 #define _FILE_OFFSET_BITS 64
 
 #include <errno.h>
+#include <fcntl.h>
 #include <limits.h>
 #include <signal.h>
-#include <stdio.h>
 #include <stdbool.h>
+#include <stdint.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <fcntl.h>
-#include <stdint.h>
 #include <unistd.h>
 #include <utime.h>
 #include <sys/stat.h>
@@ -51,19 +51,17 @@
 #endif
 
 #include "carg_parser.h"
-#include "lunzip.h"
+#include "lzip.h"
 #include "decoder.h"
 
 #if CHAR_BIT != 8
 #error "Environments where CHAR_BIT != 8 are not supported."
 #endif
 
-long long int llabs( long long int number );
-
 
 const char * const Program_name = "Lunzip";
 const char * const program_name = "lunzip";
-const char * const program_year = "2011";
+const char * const program_year = "2013";
 const char * invocation_name = 0;
 
 #ifdef O_BINARY
@@ -80,103 +78,62 @@ struct { const char * from; const char * to; } const known_extensions[] = {
 char * output_filename = 0;
 int outfd = -1;
 int verbosity = 0;
+const mode_t usr_rw = S_IRUSR | S_IWUSR;
+const mode_t all_rw = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH;
 mode_t outfd_mode = S_IRUSR | S_IWUSR;
 bool delete_output_on_interrupt = false;
 
 
-/* assure at least a minimum size for buffer `buf' */
-static void * resize_buffer( void * buf, const int min_size )
-  {
-  if( buf ) buf = realloc( buf, min_size );
-  else buf = malloc( min_size );
-  if( !buf )
-    {
-    show_error( "Not enough memory.", 0, false );
-    cleanup_and_fail( 1 );
-    }
-  return buf;
-  }
-
-
-static void show_help()
+static void show_help( void )
   {
   printf( "%s - Decompressor for lzip files.\n", Program_name );
   printf( "\nUsage: %s [options] [files]\n", invocation_name );
-  printf( "\nOptions:\n" );
-  printf( "  -h, --help                 display this help and exit\n" );
-  printf( "  -V, --version              output version information and exit\n" );
-  printf( "  -c, --stdout               send output to standard output\n" );
-  printf( "  -d, --decompress           decompress (this is the default)\n" );
-  printf( "  -f, --force                overwrite existing output files\n" );
-  printf( "  -k, --keep                 keep (don't delete) input files\n" );
-  printf( "  -o, --output=<file>        if reading stdin, place the output into <file>\n" );
-  printf( "  -q, --quiet                suppress all messages\n" );
-  printf( "  -t, --test                 test compressed file integrity\n" );
-  printf( "  -v, --verbose              be verbose (a 2nd -v gives more)\n" );
-  printf( "If no file names are given, %s decompresses from standard input to\n", program_name );
-  printf( "standard output.\n" );
-  printf( "\nReport bugs to lzip-bug@nongnu.org\n" );
-  printf( "Lunzip home page: http://www.nongnu.org/lzip/lunzip.html\n" );
+  printf( "\nOptions:\n"
+          "  -h, --help                 display this help and exit\n"
+          "  -V, --version              output version information and exit\n"
+          "  -c, --stdout               send output to standard output\n"
+          "  -d, --decompress           decompress (this is the default)\n"
+          "  -f, --force                overwrite existing output files\n"
+          "  -k, --keep                 keep (don't delete) input files\n"
+          "  -o, --output=<file>        if reading stdin, place the output into <file>\n"
+          "  -q, --quiet                suppress all messages\n"
+          "  -t, --test                 test compressed file integrity\n"
+          "  -v, --verbose              be verbose (a 2nd -v gives more)\n"
+          "If no file names are given, lunzip decompresses from standard input to\n"
+          "standard output.\n"
+          "\nExit status: 0 for a normal exit, 1 for environmental problems (file\n"
+          "not found, invalid flags, I/O errors, etc), 2 to indicate a corrupt or\n"
+          "invalid input file, 3 for an internal consistency error (eg, bug) which\n"
+          "caused lunzip to panic.\n"
+          "\nReport bugs to lzip-bug@nongnu.org\n"
+          "Lunzip home page: http://www.nongnu.org/lzip/lunzip.html\n" );
   }
 
 
-static void show_version()
+static void show_version( void )
   {
   printf( "%s %s\n", Program_name, PROGVERSION );
   printf( "Copyright (C) %s Antonio Diaz Diaz.\n", program_year );
-  printf( "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n" );
-  printf( "This is free software: you are free to change and redistribute it.\n" );
-  printf( "There is NO WARRANTY, to the extent permitted by law.\n" );
+  printf( "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n"
+          "This is free software: you are free to change and redistribute it.\n"
+          "There is NO WARRANTY, to the extent permitted by law.\n" );
   }
 
 
-static const char * format_num( long long num )
+static void show_header( const File_header header )
   {
   const char * const prefix[8] =
     { "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi", "Yi" };
-  enum { buf_size = 16, factor = 1024 };
-  static char buf[buf_size];
-  const char *p = "";
-  int i;
+  enum { factor = 1024 };
+  const char * p = "";
+  const char * np = "  ";
+  unsigned num = Fh_get_dictionary_size( header ), i;
+  bool exact = ( num % factor == 0 );
 
-  for( i = 0; i < 8 && ( llabs( num ) > 9999 ||
-       ( llabs( num ) >= factor && num % factor == 0 ) ); ++i )
-    { num /= factor; p = prefix[i]; }
-  snprintf( buf, buf_size, "%lld %s", num, p );
-  return buf;
-  }
-
-
-static int open_instream( const char * const name,
-                          struct stat * const in_statsp, const bool to_stdout )
-  {
-  int infd = -1;
-  infd = open( name, O_RDONLY | o_binary );
-  if( infd < 0 )
-    {
-    if( verbosity >= 0 )
-      fprintf( stderr, "%s: Can't open input file `%s': %s.\n",
-               program_name, name, strerror( errno ) );
-    }
-  else
-    {
-    const int i = fstat( infd, in_statsp );
-    const mode_t mode = in_statsp->st_mode;
-    const bool can_read = ( i == 0 &&
-                            ( S_ISBLK( mode ) || S_ISCHR( mode ) ||
-                              S_ISFIFO( mode ) || S_ISSOCK( mode ) ) );
-    if( i != 0 || ( !S_ISREG( mode ) && ( !to_stdout || !can_read ) ) )
-      {
-      if( verbosity >= 0 )
-        fprintf( stderr, "%s: Input file `%s' is not a regular file%s.\n",
-                 program_name, name,
-                 ( can_read && !to_stdout ) ?
-                 " and `--stdout' was not specified" : "" );
-      close( infd );
-      infd = -1;
-      }
-    }
-  return infd;
+  for( i = 0; i < 8 && ( num > 9999 || ( exact && num >= factor ) ); ++i )
+    { num /= factor; if( num % factor != 0 ) exact = false;
+      p = prefix[i]; np = ""; }
+  fprintf( stderr, "dictionary size %s%4u %sB.  ", np, num, p );
   }
 
 
@@ -194,9 +151,55 @@ static int extension_index( const char * const name )
   }
 
 
-static void set_d_outname( const char * const name )
+static int open_instream( const char * const name, struct stat * const in_statsp,
+                          const bool testing, const bool to_stdout )
+  {
+  int infd = open( name, O_RDONLY | o_binary );
+  if( infd < 0 )
+    {
+    if( verbosity >= 0 )
+      fprintf( stderr, "%s: Can't open input file '%s': %s.\n",
+               program_name, name, strerror( errno ) );
+    }
+  else
+    {
+    const int i = fstat( infd, in_statsp );
+    const mode_t mode = in_statsp->st_mode;
+    const bool can_read = ( i == 0 &&
+                            ( S_ISBLK( mode ) || S_ISCHR( mode ) ||
+                              S_ISFIFO( mode ) || S_ISSOCK( mode ) ) );
+    const bool no_ofile = to_stdout || testing;
+    if( i != 0 || ( !S_ISREG( mode ) && ( !can_read || !no_ofile ) ) )
+      {
+      if( verbosity >= 0 )
+        fprintf( stderr, "%s: Input file '%s' is not a regular file%s.\n",
+                 program_name, name,
+                 ( can_read && !no_ofile ) ?
+                 " and '--stdout' was not specified" : "" );
+      close( infd );
+      infd = -1;
+      }
+    }
+  return infd;
+  }
+
+
+/* assure at least a minimum size for buffer 'buf' */
+static void * resize_buffer( void * buf, const int min_size )
+  {
+  if( buf ) buf = realloc( buf, min_size );
+  else buf = malloc( min_size );
+  if( !buf )
+    {
+    show_error( "Not enough memory.", 0, false );
+    cleanup_and_fail( 1 );
+    }
+  return buf;
+  }
+
+
+static void set_d_outname( const char * const name, const int i )
   {
-  const int i = extension_index( name );
   if( i >= 0 )
     {
     const char * const from = known_extensions[i].from;
@@ -213,8 +216,8 @@ static void set_d_outname( const char * const name )
   output_filename = resize_buffer( output_filename, strlen( name ) + 4 + 1 );
   strcpy( output_filename, name );
   strcat( output_filename, ".out" );
-  if( verbosity >= 0 )
-    fprintf( stderr, "%s: Can't guess original name for `%s' -- using `%s'.\n",
+  if( verbosity >= 1 )
+    fprintf( stderr, "%s: Can't guess original name for '%s' -- using '%s'.\n",
              program_name, name, output_filename );
   }
 
@@ -228,10 +231,10 @@ static bool open_outstream( const bool force )
   if( outfd < 0 && verbosity >= 0 )
     {
     if( errno == EEXIST )
-      fprintf( stderr, "%s: Output file %s already exists, skipping.\n",
+      fprintf( stderr, "%s: Output file '%s' already exists, skipping.\n",
                program_name, output_filename );
     else
-      fprintf( stderr, "%s: Can't create output file `%s': %s.\n",
+      fprintf( stderr, "%s: Can't create output file '%s': %s.\n",
                program_name, output_filename, strerror( errno ) );
     }
   return ( outfd >= 0 );
@@ -244,10 +247,10 @@ void cleanup_and_fail( const int retval )
     {
     delete_output_on_interrupt = false;
     if( verbosity >= 0 )
-      fprintf( stderr, "%s: Deleting output file `%s', if it exists.\n",
+      fprintf( stderr, "%s: Deleting output file '%s', if it exists.\n",
                program_name, output_filename );
     if( outfd >= 0 ) { close( outfd ); outfd = -1; }
-    if( remove( output_filename ) != 0 )
+    if( remove( output_filename ) != 0 && errno != ENOENT )
       show_error( "WARNING: deletion of output file (apparently) failed.", 0, false );
     }
   exit( retval );
@@ -257,54 +260,54 @@ void cleanup_and_fail( const int retval )
      /* Set permissions, owner and times. */
 static void close_and_set_permissions( const struct stat * const in_statsp )
   {
-  bool error = false;
+  bool warning = false;
   if( in_statsp )
     {
+    /* fchown will in many cases return with EPERM, which can be safely ignored. */
     if( ( fchown( outfd, in_statsp->st_uid, in_statsp->st_gid ) != 0 &&
           errno != EPERM ) ||
-        fchmod( outfd, in_statsp->st_mode ) != 0 )
-      error = true;
-    /* fchown will in many cases return with EPERM, which can be safely ignored. */
+        fchmod( outfd, in_statsp->st_mode ) != 0 ) warning = true;
     }
-  if( close( outfd ) == 0 ) outfd = -1;
-  else cleanup_and_fail( 1 );
+  if( close( outfd ) != 0 ) cleanup_and_fail( 1 );
+  outfd = -1;
   delete_output_on_interrupt = false;
-  if( !in_statsp ) return;
-  if( !error )
+  if( in_statsp )
     {
     struct utimbuf t;
     t.actime = in_statsp->st_atime;
     t.modtime = in_statsp->st_mtime;
-    if( utime( output_filename, &t ) != 0 ) error = true;
+    if( utime( output_filename, &t ) != 0 ) warning = true;
     }
-  if( error )
-    {
+  if( warning && verbosity >= 1 )
     show_error( "Can't change output file attributes.", 0, false );
-    cleanup_and_fail( 1 );
-    }
   }
 
 
 static int decompress( const int infd, struct Pretty_print * const pp,
                        const bool testing )
   {
-  long long partial_file_pos = 0;
+  unsigned long long partial_file_pos = 0;
   struct Range_decoder rdec;
-  int retval = 0, i, result;
+  int retval = 0;
   bool first_member;
-  Rd_init( &rdec, infd );
-
-  for( first_member = true; ; first_member = false, Pp_reset( pp ) )
+  if( !Rd_init( &rdec, infd ) )
     {
+    show_error( "Not enough memory. Find a machine with more memory.", 0, false );
+    cleanup_and_fail( 1 );
+    }
+
+  for( first_member = true; ; first_member = false )
+    {
+    int result;
     File_header header;
     struct LZ_decoder decoder;
     Rd_reset_member_position( &rdec );
-    for( i = 0; i < Fh_size; ++i )
-      header[i] = Rd_get_byte( &rdec );
-    if( Rd_finished( &rdec ) )		/* End Of File */
+    Rd_read_data( &rdec, header, Fh_size );
+    if( Rd_finished( &rdec ) )			/* End Of File */
       {
       if( first_member )
-        { Pp_show_msg( pp, "Error reading member header" ); retval = 1; }
+        { Pp_show_msg( pp, "File ends unexpectedly at member header" );
+          retval = 2; }
       break;
       }
     if( !Fh_verify_magic( header ) )
@@ -326,16 +329,14 @@ static int decompress( const int infd, struct Pretty_print * const pp,
       { Pp_show_msg( pp, "Invalid dictionary size in member header" );
         retval = 2; break; }
 
-    if( verbosity >= 1 )
-      {
-      Pp_show_msg( pp, 0 );
-      if( verbosity >= 2 )
-        fprintf( stderr, "version %d, dictionary size %7sB.  ",
-                 Fh_version( header ),
-                 format_num( Fh_get_dictionary_size( header ) ) );
-      }
-    LZd_init( &decoder, header, &rdec, outfd );
+    if( verbosity >= 2 || ( verbosity == 1 && first_member ) )
+      { Pp_show_msg( pp, 0 ); if( verbosity >= 3 ) show_header( header ); }
 
+    if( !LZd_init( &decoder, header, &rdec, outfd ) )
+      {
+      show_error( "Not enough memory. Find a machine with more memory.", 0, false );
+      cleanup_and_fail( 1 );
+      }
     result = LZd_decode_member( &decoder, pp );
     partial_file_pos += Rd_member_position( &rdec );
     LZd_free( &decoder );
@@ -345,31 +346,32 @@ static int decompress( const int infd, struct Pretty_print * const pp,
         {
         Pp_show_msg( pp, 0 );
         if( result == 2 )
-          fprintf( stderr, "File ends unexpectedly at pos %lld\n",
+          fprintf( stderr, "File ends unexpectedly at pos %llu\n",
                    partial_file_pos );
         else
-          fprintf( stderr, "Decoder error at pos %lld\n", partial_file_pos );
+          fprintf( stderr, "Decoder error at pos %llu\n", partial_file_pos );
         }
       retval = 2; break;
       }
-    if( verbosity >= 1 )
-      { if( testing ) fprintf( stderr, "ok\n" );
-        else fprintf( stderr, "done\n" ); }
+    if( verbosity >= 2 )
+      { fprintf( stderr, testing ? "ok\n" : "done\n" ); Pp_reset( pp ); }
     }
   Rd_free( &rdec );
+  if( verbosity == 1 && retval == 0 )
+    fprintf( stderr, testing ? "ok\n" : "done\n" );
   return retval;
   }
 
 
 void signal_handler( int sig )
   {
-  sig = 0;			/* keep compiler happy */
+  if( sig ) {}				/* keep compiler happy */
   show_error( "Control-C or similar caught, quitting.", 0, false );
   cleanup_and_fail( 1 );
   }
 
 
-static void set_signals()
+static void set_signals( void )
   {
   signal( SIGHUP, signal_handler );
   signal( SIGINT, signal_handler );
@@ -378,45 +380,26 @@ static void set_signals()
 
 
 void Pp_init( struct Pretty_print * const pp, const char * const filenames[],
-              const int num_filenames, const int v )
+              const int num_filenames )
   {
-  unsigned int stdin_name_len;
+  unsigned stdin_name_len;
   int i;
-  pp->name_ = 0;
+  pp->name = 0;
   pp->stdin_name = "(stdin)";
   pp->longest_name = 0;
-  pp->verbosity = v;
   pp->first_post = false;
   stdin_name_len = strlen( pp->stdin_name );
 
   for( i = 0; i < num_filenames; ++i )
     {
     const char * const s = filenames[i];
-    const int len = ( !strcmp( s, "-" ) ? stdin_name_len : strlen( s ) );
+    const int len = ( (strcmp( s, "-" ) == 0) ? stdin_name_len : strlen( s ) );
     if( len > pp->longest_name ) pp->longest_name = len;
     }
   if( pp->longest_name == 0 ) pp->longest_name = stdin_name_len;
   }
 
 
-void Pp_show_msg( struct Pretty_print * const pp, const char * const msg )
-  {
-  if( verbosity >= 0 )
-    {
-    if( pp->first_post )
-      {
-      int i, len;
-      pp->first_post = false;
-      fprintf( stderr, "  %s: ", pp->name_ );
-      len = pp->longest_name - strlen( pp->name_ );
-      for( i = 0; i < len; ++i ) fprintf( stderr, " " );
-      if( !msg ) fflush( stderr );
-      }
-    if( msg ) fprintf( stderr, "%s.\n", msg );
-    }
-  }
-
-
 void show_error( const char * const msg, const int errcode, const bool help )
   {
   if( verbosity >= 0 )
@@ -427,8 +410,8 @@ void show_error( const char * const msg, const int errcode, const bool help )
       if( errcode > 0 ) fprintf( stderr, ": %s", strerror( errcode ) );
       fprintf( stderr, "\n" );
       }
-    if( help && invocation_name && invocation_name[0] )
-      fprintf( stderr, "Try `%s --help' for more information.\n",
+    if( help )
+      fprintf( stderr, "Try '%s --help' for more information.\n",
                invocation_name );
     }
   }
@@ -442,45 +425,6 @@ void internal_error( const char * const msg )
   }
 
 
-/* Returns the number of bytes really read.
-   If (returned value < size) and (errno == 0), means EOF was reached.
-*/
-int readblock( const int fd, uint8_t * const buf, const int size )
-  {
-  int rest = size;
-  while( true )
-    {
-    int n;
-    errno = 0;
-    if( rest <= 0 ) break;
-    n = read( fd, buf + size - rest, rest );
-    if( n > 0 ) rest -= n;
-    else if( n == 0 ) break;
-    else if( errno != EINTR && errno != EAGAIN ) break;
-    }
-  return ( rest > 0 ) ? size - rest : size;
-  }
-
-
-/* Returns the number of bytes really written.
-   If (returned value < size), it is always an error.
-*/
-int writeblock( const int fd, const uint8_t * const buf, const int size )
-  {
-  int rest = size;
-  while( true )
-    {
-    int n;
-    errno = 0;
-    if( rest <= 0 ) break;
-    n = write( fd, buf + size - rest, rest );
-    if( n > 0 ) rest -= n;
-    else if( errno && errno != EINTR && errno != EAGAIN ) break;
-    }
-  return ( rest > 0 ) ? size - rest : size;
-  }
-
-
 int main( const int argc, const char * const argv[] )
   {
   const char * input_filename = "";
@@ -505,6 +449,7 @@ int main( const int argc, const char * const argv[] )
     { 'f', "force",           ap_no  },
     { 'h', "help",            ap_no  },
     { 'k', "keep",            ap_no  },
+    { 'n', "threads",         ap_yes },
     { 'o', "output",          ap_yes },
     { 'q', "quiet",           ap_no  },
     { 't', "test",            ap_no  },
@@ -516,6 +461,7 @@ int main( const int argc, const char * const argv[] )
 
   invocation_name = argv[0];
   CRC32_init();
+
   if( !ap_init( &parser, argc, argv, options, 0 ) )
     { show_error( "Memory exhausted.", 0, false ); return 1; }
   if( ap_error( &parser ) )				/* bad option */
@@ -525,7 +471,7 @@ int main( const int argc, const char * const argv[] )
     {
     const int code = ap_code( &parser, argind );
     const char * const arg = ap_argument( &parser, argind );
-    if( !code ) break;				/* no more options */
+    if( !code ) break;					/* no more options */
     switch( code )
       {
       case 'c': to_stdout = true; break;
@@ -533,6 +479,7 @@ int main( const int argc, const char * const argv[] )
       case 'f': force = true; break;
       case 'h': show_help(); return 0;
       case 'k': keep_input_files = true; break;
+      case 'n': break;
       case 'o': default_output_filename = arg; break;
       case 'q': verbosity = -1; break;
       case 't': testing = true; break;
@@ -543,32 +490,28 @@ int main( const int argc, const char * const argv[] )
     } /* end process options */
 
 #if defined(__MSVCRT__) || defined(__OS2__)
-  _fsetmode( stdin, "b" );
-  _fsetmode( stdout, "b" );
+  setmode( STDIN_FILENO, O_BINARY );
+  setmode( STDOUT_FILENO, O_BINARY );
 #endif
 
-  for( ; argind < ap_arguments( &parser ); ++argind )
+  if( testing )
+    outfd = -1;
+
+  num_filenames = max( 1, ap_arguments( &parser ) - argind );
+  filenames = resize_buffer( filenames, num_filenames * sizeof filenames[0] );
+  filenames[0] = "-";
+
+  for( i = 0; argind + i < ap_arguments( &parser ); ++i )
     {
-    if( strcmp( ap_argument( &parser, argind ), "-" ) )
-      filenames_given = true;
-    ++num_filenames;
-    filenames = resize_buffer( filenames, num_filenames * sizeof (char *) );
-    filenames[num_filenames-1] = ap_argument( &parser, argind );
+    filenames[i] = ap_argument( &parser, argind + i );
+    if( strcmp( filenames[i], "-" ) != 0 ) filenames_given = true;
     }
 
-  if( num_filenames == 0 )
-    {
-    ++num_filenames;
-    filenames = resize_buffer( filenames, sizeof (char *) );
-    filenames[num_filenames-1] = "-";
-    }
   if( !to_stdout && !testing &&
       ( filenames_given || default_output_filename[0] ) )
     set_signals();
 
-  Pp_init( &pp, filenames, num_filenames, verbosity );
-  if( testing )
-    outfd = -1;
+  Pp_init( &pp, filenames, num_filenames );
 
   output_filename = resize_buffer( output_filename, 1 );
   for( i = 0; i < num_filenames; ++i )
@@ -578,7 +521,7 @@ int main( const int argc, const char * const argv[] )
     const struct stat * in_statsp;
     output_filename[0] = 0;
 
-    if( !filenames[i][0] || !strcmp( filenames[i], "-" ) )
+    if( !filenames[i][0] || strcmp( filenames[i], "-" ) == 0 )
       {
       input_filename = "";
       infd = STDIN_FILENO;
@@ -591,10 +534,10 @@ int main( const int argc, const char * const argv[] )
           output_filename = resize_buffer( output_filename,
                                            strlen( default_output_filename ) + 1 );
           strcpy( output_filename, default_output_filename );
-          outfd_mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH;
+          outfd_mode = all_rw;
           if( !open_outstream( force ) )
             {
-            if( outfd == -1 && retval < 1 ) retval = 1;
+            if( retval < 1 ) retval = 1;
             close( infd ); infd = -1;
             continue;
             }
@@ -604,18 +547,18 @@ int main( const int argc, const char * const argv[] )
     else
       {
       input_filename = filenames[i];
-      infd = open_instream( input_filename, &in_stats, to_stdout );
+      infd = open_instream( input_filename, &in_stats, testing, to_stdout );
       if( infd < 0 ) { if( retval < 1 ) retval = 1; continue; }
       if( !testing )
         {
         if( to_stdout ) outfd = STDOUT_FILENO;
         else
           {
-          set_d_outname( input_filename );
-          outfd_mode = S_IRUSR | S_IWUSR;
+          set_d_outname( input_filename, extension_index( input_filename ) );
+          outfd_mode = usr_rw;
           if( !open_outstream( force ) )
             {
-            if( outfd == -1 && retval < 1 ) retval = 1;
+            if( retval < 1 ) retval = 1;
             close( infd ); infd = -1;
             continue;
             }
